@@ -1,32 +1,6 @@
-/*
-  Copyright 2008 Larry Gritz and the other authors and contributors.
-  All Rights Reserved.
-
-  Redistribution and use in source and binary forms, with or without
-  modification, are permitted provided that the following conditions are
-  met:
-  * Redistributions of source code must retain the above copyright
-    notice, this list of conditions and the following disclaimer.
-  * Redistributions in binary form must reproduce the above copyright
-    notice, this list of conditions and the following disclaimer in the
-    documentation and/or other materials provided with the distribution.
-  * Neither the name of the software's owners nor the names of its
-    contributors may be used to endorse or promote products derived from
-    this software without specific prior written permission.
-  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
-  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
-  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
-  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
-  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
-  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
-  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
-  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
-  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-
-  (This is the Modified BSD License)
-*/
+// Copyright 2008-present Contributors to the OpenImageIO project.
+// SPDX-License-Identifier: BSD-3-Clause
+// https://github.com/OpenImageIO/oiio/blob/master/LICENSE.md
 
 #include <cassert>
 #include <cstdio>
@@ -128,7 +102,7 @@ JpgOutput::open(const std::string& name, const ImageSpec& newspec,
                 OpenMode mode)
 {
     if (mode != Create) {
-        error("%s does not support subimages or MIP levels", format_name());
+        errorf("%s does not support subimages or MIP levels", format_name());
         return false;
     }
 
@@ -138,20 +112,20 @@ JpgOutput::open(const std::string& name, const ImageSpec& newspec,
 
     // Check for things this format doesn't support
     if (m_spec.width < 1 || m_spec.height < 1) {
-        error("Image resolution must be at least 1x1, you asked for %d x %d",
-              m_spec.width, m_spec.height);
+        errorf("Image resolution must be at least 1x1, you asked for %d x %d",
+               m_spec.width, m_spec.height);
         return false;
     }
     if (m_spec.depth < 1)
         m_spec.depth = 1;
     if (m_spec.depth > 1) {
-        error("%s does not support volume images (depth > 1)", format_name());
+        errorf("%s does not support volume images (depth > 1)", format_name());
         return false;
     }
 
     m_fd = Filesystem::fopen(name, "wb");
     if (m_fd == NULL) {
-        error("Unable to open file \"%s\"", name.c_str());
+        errorf("Could not open \"%s\"", name);
         return false;
     }
 
@@ -200,9 +174,12 @@ JpgOutput::open(const std::string& name, const ImageSpec& newspec,
         // Careful -- jpeg_set_defaults overwrites density
         resmeta_to_density();
         DBG std::cout << "out open: set_defaults\n";
-        int quality = newspec.get_int_attribute("CompressionQuality", 98);
-        jpeg_set_quality(&m_cinfo, quality, TRUE);  // baseline values
-        DBG std::cout << "out open: set_quality\n";
+
+        auto compqual = m_spec.decode_compression_metadata("jpeg", 98);
+        if (Strutil::iequals(compqual.first, "jpeg"))
+            jpeg_set_quality(&m_cinfo, clamp(compqual.second, 1, 100), TRUE);
+        else
+            jpeg_set_quality(&m_cinfo, 98, TRUE);  // not jpeg? default qual
 
         if (m_cinfo.input_components == 3) {
             std::string subsampling = m_spec.get_string_attribute(
@@ -293,9 +270,9 @@ JpgOutput::open(const std::string& name, const ImageSpec& newspec,
             if ((unsigned int)(num_markers * MAX_DATA_BYTES_IN_MARKER)
                 != icc_profile_length)
                 num_markers++;
-            int curr_marker = 1; /* per spec, count strarts at 1*/
-            std::vector<unsigned char> profile(MAX_DATA_BYTES_IN_MARKER
-                                               + ICC_HEADER_SIZE);
+            int curr_marker     = 1; /* per spec, count strarts at 1*/
+            size_t profile_size = MAX_DATA_BYTES_IN_MARKER + ICC_HEADER_SIZE;
+            std::vector<unsigned char> profile(profile_size);
             while (icc_profile_length > 0) {
                 // length of profile to put in this marker
                 unsigned int length
@@ -303,7 +280,7 @@ JpgOutput::open(const std::string& name, const ImageSpec& newspec,
                                (unsigned int)MAX_DATA_BYTES_IN_MARKER);
                 icc_profile_length -= length;
                 // Write the JPEG marker header (APP2 code and marker length)
-                strcpy((char*)&profile[0], "ICC_PROFILE");
+                strncpy((char*)&profile[0], "ICC_PROFILE", profile_size);
                 profile[11] = 0;
                 profile[12] = curr_marker;
                 profile[13] = (unsigned char)num_markers;
@@ -370,12 +347,11 @@ JpgOutput::write_scanline(int y, int z, TypeDesc format, const void* data,
 {
     y -= m_spec.y;
     if (y != m_next_scanline) {
-        error("Attempt to write scanlines out of order to %s",
-              m_filename.c_str());
+        errorf("Attempt to write scanlines out of order to %s", m_filename);
         return false;
     }
     if (y >= (int)m_cinfo.image_height) {
-        error("Attempt to write too many scanlines to %s", m_filename.c_str());
+        errorf("Attempt to write too many scanlines to %s", m_filename);
         return false;
     }
     assert(y == (int)m_cinfo.next_scanline);
