@@ -1,32 +1,6 @@
-/*
-  Copyright 2008 Larry Gritz and the other authors and contributors.
-  All Rights Reserved.
-
-  Redistribution and use in source and binary forms, with or without
-  modification, are permitted provided that the following conditions are
-  met:
-  * Redistributions of source code must retain the above copyright
-    notice, this list of conditions and the following disclaimer.
-  * Redistributions in binary form must reproduce the above copyright
-    notice, this list of conditions and the following disclaimer in the
-    documentation and/or other materials provided with the distribution.
-  * Neither the name of the software's owners nor the names of its
-    contributors may be used to endorse or promote products derived from
-    this software without specific prior written permission.
-  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
-  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
-  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
-  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
-  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
-  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
-  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
-  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
-  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-
-  (This is the Modified BSD License)
-*/
+// Copyright 2008-present Contributors to the OpenImageIO project.
+// SPDX-License-Identifier: BSD-3-Clause
+// https://github.com/OpenImageIO/oiio/blob/master/LICENSE.md
 
 /// \file
 /// ImageBufAlgo functions for filtered transformations
@@ -52,6 +26,17 @@ OIIO_NAMESPACE_BEGIN
 
 namespace {
 
+// Define a templated Accumulator type that's float, except in the case
+// of double input, in which case it's double.
+template<typename T> struct Accum_t {
+    typedef float type;
+};
+template<> struct Accum_t<double> {
+    typedef double type;
+};
+
+
+
 // Poor man's Dual2<float> makes it easy to compute with differentials. For
 // a rich man's implementation and full documentation, see
 // OpenShadingLanguage (dual2.h).
@@ -179,7 +164,7 @@ filtered_sample(const ImageBuf& src, float s, float t, float dsdx, float dtdx,
                                           (int)ceilf(t + filterrad_t), 0, 1,
                                           wrap);
     int nc     = src.nchannels();
-    float* sum = ALLOCA(float, nc);
+    float* sum = OIIO_ALLOCA(float, nc);
     memset(sum, 0, nc * sizeof(float));
     float total_w = 0.0f;
     for (; !samp.done(); ++samp) {
@@ -232,7 +217,6 @@ resize_(ImageBuf& dst, const ImageBuf& src, Filter2D* filter, ROI roi,
         float dstfh          = float(dstspec.full_height);
         float dstpixelwidth  = 1.0f / dstfw;
         float dstpixelheight = 1.0f / dstfh;
-        float* pel           = ALLOCA(float, nchannels);
         float filterrad      = filter->width() / 2.0f;
 
         // radi,radj is the filter radius, as an integer, in source pixels.  We
@@ -242,7 +226,7 @@ resize_(ImageBuf& dst, const ImageBuf& src, Filter2D* filter, ROI roi,
         int xtaps           = 2 * radi + 1;
         int ytaps           = 2 * radj + 1;
         bool separable      = filter->separable();
-        float* yfiltval     = ALLOCA(float, ytaps);
+        float* yfiltval     = OIIO_ALLOCA(float, ytaps);
         float* xfiltval_all = NULL;
         if (separable) {
             // For separable filters, horizontal tap weights will be the same
@@ -250,15 +234,13 @@ resize_(ImageBuf& dst, const ImageBuf& src, Filter2D* filter, ROI roi,
             // x position we'll need. We do the same thing in y, but row by row
             // inside the loop (since we never revisit a y row). This
             // substantially speeds up resize.
-            xfiltval_all = ALLOCA(float, xtaps* roi.width());
+            xfiltval_all = OIIO_ALLOCA(float, xtaps* roi.width());
             for (int x = roi.xbegin; x < roi.xend; ++x) {
                 float* xfiltval = xfiltval_all + (x - roi.xbegin) * xtaps;
                 float s         = (x - dstfx + 0.5f) * dstpixelwidth;
                 float src_xf    = srcfx + s * srcfw;
                 int src_x;
-                float src_xf_frac = floorfrac(src_xf, &src_x);
-                for (int c = 0; c < nchannels; ++c)
-                    pel[c] = 0.0f;
+                float src_xf_frac   = floorfrac(src_xf, &src_x);
                 float totalweight_x = 0.0f;
                 for (int i = 0; i < xtaps; ++i) {
                     float w = filter->xfilt(
@@ -283,6 +265,11 @@ resize_(ImageBuf& dst, const ImageBuf& src, Filter2D* filter, ROI roi,
     std::cerr << "separable filter\n";
 #endif
 
+        // Accumulate the weighted results in pel[]. We select a type big
+        // enough to hold with required precision.
+        typedef typename Accum_t<DSTTYPE>::type Acc_t;
+        Acc_t* pel = OIIO_ALLOCA(Acc_t, nchannels);
+
 #define USE_SPECIAL 0
 #if USE_SPECIAL
         // Special case: src and dst are local memory, float buffers, and we're
@@ -460,7 +447,7 @@ get_resize_filter(string_view filtername, float fwidth, ImageBuf& dst,
         }
     }
     if (!filter) {
-        dst.error("Filter \"%s\" not recognized", filtername);
+        dst.errorf("Filter \"%s\" not recognized", filtername);
     }
     return filter;
 }
@@ -473,8 +460,7 @@ ImageBufAlgo::resize(ImageBuf& dst, const ImageBuf& src, Filter2D* filter,
 {
     pvt::LoggedTimer logtime("IBA::resize");
     if (!IBAprep(roi, &dst, &src,
-                 IBAprep_REQUIRE_SAME_NCHANNELS | IBAprep_NO_SUPPORT_VOLUME
-                     | IBAprep_NO_COPY_ROI_FULL))
+                 IBAprep_NO_SUPPORT_VOLUME | IBAprep_NO_COPY_ROI_FULL))
         return false;
 
     // Set up a shared pointer with custom deleter to make sure any
@@ -507,8 +493,7 @@ ImageBufAlgo::resize(ImageBuf& dst, const ImageBuf& src, string_view filtername,
 {
     pvt::LoggedTimer logtime("IBA::resize");
     if (!IBAprep(roi, &dst, &src,
-                 IBAprep_REQUIRE_SAME_NCHANNELS | IBAprep_NO_SUPPORT_VOLUME
-                     | IBAprep_NO_COPY_ROI_FULL))
+                 IBAprep_NO_SUPPORT_VOLUME | IBAprep_NO_COPY_ROI_FULL))
         return false;
     const ImageSpec& srcspec(src.spec());
     const ImageSpec& dstspec(dst.spec());
@@ -536,7 +521,7 @@ ImageBufAlgo::resize(const ImageBuf& src, Filter2D* filter, ROI roi,
     ImageBuf result;
     bool ok = resize(result, src, filter, roi, nthreads);
     if (!ok && !result.has_error())
-        result.error("ImageBufAlgo::resize() error");
+        result.errorf("ImageBufAlgo::resize() error");
     return result;
 }
 
@@ -548,7 +533,7 @@ ImageBufAlgo::resize(const ImageBuf& src, string_view filtername,
     ImageBuf result;
     bool ok = resize(result, src, filtername, filterwidth, roi, nthreads);
     if (!ok && !result.has_error())
-        result.error("ImageBufAlgo::resize() error");
+        result.errorf("ImageBufAlgo::resize() error");
     return result;
 }
 
@@ -560,8 +545,7 @@ ImageBufAlgo::fit(ImageBuf& dst, const ImageBuf& src, Filter2D* filter,
 {
     // No time logging, it will be accounted in the underlying warp/resize
     if (!IBAprep(roi, &dst, &src,
-                 IBAprep_REQUIRE_SAME_NCHANNELS | IBAprep_NO_SUPPORT_VOLUME
-                     | IBAprep_NO_COPY_ROI_FULL))
+                 IBAprep_NO_SUPPORT_VOLUME | IBAprep_NO_COPY_ROI_FULL))
         return false;
 
     const ImageSpec& srcspec(src.spec());
@@ -661,8 +645,7 @@ ImageBufAlgo::fit(ImageBuf& dst, const ImageBuf& src, string_view filtername,
 {
     pvt::LoggedTimer logtime("IBA::fit");
     if (!IBAprep(roi, &dst, &src,
-                 IBAprep_REQUIRE_SAME_NCHANNELS | IBAprep_NO_SUPPORT_VOLUME
-                     | IBAprep_NO_COPY_ROI_FULL))
+                 IBAprep_NO_SUPPORT_VOLUME | IBAprep_NO_COPY_ROI_FULL))
         return false;
     const ImageSpec& srcspec(src.spec());
     const ImageSpec& dstspec(dst.spec());
@@ -689,7 +672,7 @@ ImageBufAlgo::fit(const ImageBuf& src, Filter2D* filter, bool exact, ROI roi,
     ImageBuf result;
     bool ok = fit(result, src, filter, exact, roi, nthreads);
     if (!ok && !result.has_error())
-        result.error("ImageBufAlgo::fit() error");
+        result.errorf("ImageBufAlgo::fit() error");
     return result;
 }
 
@@ -701,7 +684,7 @@ ImageBufAlgo::fit(const ImageBuf& src, string_view filtername,
     ImageBuf result;
     bool ok = fit(result, src, filtername, filterwidth, exact, roi, nthreads);
     if (!ok && !result.has_error())
-        result.error("ImageBufAlgo::fit() error");
+        result.errorf("ImageBufAlgo::fit() error");
     return result;
 }
 
@@ -731,7 +714,7 @@ resample_(ImageBuf& dst, const ImageBuf& src, bool interpolate, ROI roi,
         float dstfh          = dstspec.full_height;
         float dstpixelwidth  = 1.0f / dstfw;
         float dstpixelheight = 1.0f / dstfh;
-        float* pel           = ALLOCA(float, nchannels);
+        float* pel           = OIIO_ALLOCA(float, nchannels);
 
         ImageBuf::Iterator<DSTTYPE> out(dst, roi);
         ImageBuf::ConstIterator<SRCTYPE> srcpel(src);
@@ -787,8 +770,8 @@ ImageBufAlgo::resample(ImageBuf& dst, const ImageBuf& src, bool interpolate,
 {
     pvt::LoggedTimer logtime("IBA::resample");
     if (!IBAprep(roi, &dst, &src,
-                 IBAprep_REQUIRE_SAME_NCHANNELS | IBAprep_NO_SUPPORT_VOLUME
-                     | IBAprep_NO_COPY_ROI_FULL | IBAprep_SUPPORT_DEEP))
+                 IBAprep_NO_SUPPORT_VOLUME | IBAprep_NO_COPY_ROI_FULL
+                     | IBAprep_SUPPORT_DEEP))
         return false;
 
     if (dst.deep()) {
@@ -831,7 +814,7 @@ ImageBufAlgo::resample(const ImageBuf& src, bool interpolate, ROI roi,
     ImageBuf result;
     bool ok = resample(result, src, interpolate, roi, nthreads);
     if (!ok && !result.has_error())
-        result.error("ImageBufAlgo::resample() error");
+        result.errorf("ImageBufAlgo::resample() error");
     return result;
 }
 
@@ -867,7 +850,7 @@ warp_(ImageBuf& dst, const ImageBuf& src, const Imath::M33f& M,
 {
     ImageBufAlgo::parallel_image(roi, nthreads, [&](ROI roi) {
         int nc     = dst.nchannels();
-        float* pel = ALLOCA(float, nc);
+        float* pel = OIIO_ALLOCA(float, nc);
         memset(pel, 0, nc * sizeof(float));
         Imath::M33f Minv = M.inverse();
         ImageBuf::Iterator<DSTTYPE> out(dst, roi);
@@ -948,7 +931,7 @@ ImageBufAlgo::warp(ImageBuf& dst, const ImageBuf& src, const Imath::M33f& M,
         }
     }
     if (!filter) {
-        dst.error("Filter \"%s\" not recognized", filtername);
+        dst.errorf("Filter \"%s\" not recognized", filtername);
         return false;
     }
 
@@ -965,7 +948,7 @@ ImageBufAlgo::warp(const ImageBuf& src, const Imath::M33f& M,
     ImageBuf result;
     bool ok = warp(result, src, M, filter, recompute_roi, wrap, roi, nthreads);
     if (!ok && !result.has_error())
-        result.error("ImageBufAlgo::warp() error");
+        result.errorf("ImageBufAlgo::warp() error");
     return result;
 }
 
@@ -981,7 +964,7 @@ ImageBufAlgo::warp(const ImageBuf& src, const Imath::M33f& M,
     bool ok = warp(result, src, M, filtername, filterwidth, recompute_roi, wrap,
                    roi, nthreads);
     if (!ok && !result.has_error())
-        result.error("ImageBufAlgo::warp() error");
+        result.errorf("ImageBufAlgo::warp() error");
     return result;
 }
 
@@ -1058,7 +1041,7 @@ ImageBufAlgo::rotate(const ImageBuf& src, float angle, float center_x,
     bool ok = rotate(result, src, angle, center_x, center_y, filter,
                      recompute_roi, roi, nthreads);
     if (!ok && !result.has_error())
-        result.error("ImageBufAlgo::rotate() error");
+        result.errorf("ImageBufAlgo::rotate() error");
     return result;
 }
 
@@ -1073,7 +1056,7 @@ ImageBufAlgo::rotate(const ImageBuf& src, float angle, float center_x,
     bool ok = rotate(result, src, angle, center_x, center_y, filtername,
                      filterwidth, recompute_roi, roi, nthreads);
     if (!ok && !result.has_error())
-        result.error("ImageBufAlgo::rotate() error");
+        result.errorf("ImageBufAlgo::rotate() error");
     return result;
 }
 
@@ -1086,7 +1069,7 @@ ImageBufAlgo::rotate(const ImageBuf& src, float angle, Filter2D* filter,
     ImageBuf result;
     bool ok = rotate(result, src, angle, filter, recompute_roi, roi, nthreads);
     if (!ok && !result.has_error())
-        result.error("ImageBufAlgo::rotate() error");
+        result.errorf("ImageBufAlgo::rotate() error");
     return result;
 }
 
@@ -1101,7 +1084,7 @@ ImageBufAlgo::rotate(const ImageBuf& src, float angle, string_view filtername,
     bool ok = rotate(result, src, angle, filtername, filterwidth, recompute_roi,
                      roi, nthreads);
     if (!ok && !result.has_error())
-        result.error("ImageBufAlgo::rotate() error");
+        result.errorf("ImageBufAlgo::rotate() error");
     return result;
 }
 
