@@ -1,32 +1,6 @@
-/*
-  Copyright 2008 Larry Gritz and the other authors and contributors.
-  All Rights Reserved.
-
-  Redistribution and use in source and binary forms, with or without
-  modification, are permitted provided that the following conditions are
-  met:
-  * Redistributions of source code must retain the above copyright
-    notice, this list of conditions and the following disclaimer.
-  * Redistributions in binary form must reproduce the above copyright
-    notice, this list of conditions and the following disclaimer in the
-    documentation and/or other materials provided with the distribution.
-  * Neither the name of the software's owners nor the names of its
-    contributors may be used to endorse or promote products derived from
-    this software without specific prior written permission.
-  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
-  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
-  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
-  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
-  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
-  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
-  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
-  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
-  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-
-  (This is the Modified BSD License)
-*/
+// Copyright 2008-present Contributors to the OpenImageIO project.
+// SPDX-License-Identifier: BSD-3-Clause
+// https://github.com/OpenImageIO/oiio/blob/master/LICENSE.md
 
 
 // Filter results comparison
@@ -36,7 +10,9 @@
 //     - prman 16.0 txmake
 //
 // box: oiio, prman, katana, imagemagick match
-// lanczos3: oiio, katana, imagemagick match.  prman is far sharper (perhaps lanczos2?)
+// lanczos3: oiio, katana, imagemagick match.  prman is far sharper
+//   (perhaps lanczos2?). Note that Nuke calls this filter "lanczos6" (they
+//   measure full width).
 // sinc: oiio, prman match.  Katana is slighly softer. imagemagick is much softer
 // blackman harris:  all differ. In order of decreasing sharpness... imagemagick, oiio, prman
 // catrom: oiio, imagemagick, prman match
@@ -848,6 +824,7 @@ FilterDesc filter1d_list[] = {
     { "blackman-harris", 1,   3,    false,    true,     true },
     { "sinc",            1,   4,    false,    true,     true },
     { "lanczos3",        1,   6,    false,    true,     true },
+    { "nuke-lanczos6",   1,   6,    false,    true,     true },
     { "mitchell",        1,   4,    false,    true,     true },
     { "bspline",         1,   4,    false,    true,     true },
     { "cubic",           1,   4,    false,    true,     true },
@@ -864,11 +841,17 @@ Filter1D::num_filters()
     return sizeof(filter1d_list) / sizeof(filter1d_list[0]);
 }
 
+const FilterDesc&
+Filter1D::get_filterdesc(int filternum)
+{
+    ASSERT(filternum >= 0 && filternum < num_filters());
+    return filter1d_list[filternum];
+}
+
 void
 Filter1D::get_filterdesc(int filternum, FilterDesc* filterdesc)
 {
-    ASSERT(filternum >= 0 && filternum < num_filters());
-    *filterdesc = filter1d_list[filternum];
+    *filterdesc = get_filterdesc(filternum);
 }
 
 
@@ -894,7 +877,8 @@ Filter1D::create(string_view filtername, float width)
         return new FilterBlackmanHarris1D(width);
     if (filtername == "sinc")
         return new FilterSinc1D(width);
-    if (filtername == "lanczos3" || filtername == "lanczos")
+    if (filtername == "lanczos3" || filtername == "lanczos"
+        || filtername == "nuke-lanczos6")
         return new FilterLanczos3_1D(width);
     if (filtername == "mitchell")
         return new FilterMitchell1D(width);
@@ -933,6 +917,7 @@ static FilterDesc filter2d_list[] = {
     { "sinc",            2,   4,    false,    true,     true  },
     { "lanczos3",        2,   6,    false,    true,     true  },
     { "radial-lanczos3", 2,   6,    false,    true,     false },
+    { "nuke-lanczos6",   2,   6,    false,    true,     false },
     { "mitchell",        2,   4,    false,    true,     true  },
     { "bspline",         2,   4,    false,    true,     true  },
     { "disk",            2,   1,    false,    true,     false },
@@ -949,11 +934,17 @@ Filter2D::num_filters()
     return sizeof(filter2d_list) / sizeof(filter2d_list[0]);
 }
 
+const FilterDesc&
+Filter2D::get_filterdesc(int filternum)
+{
+    ASSERT(filternum >= 0 && filternum < num_filters());
+    return filter2d_list[filternum];
+}
+
 void
 Filter2D::get_filterdesc(int filternum, FilterDesc* filterdesc)
 {
-    ASSERT(filternum >= 0 && filternum < num_filters());
-    *filterdesc = filter2d_list[filternum];
+    *filterdesc = get_filterdesc(filternum);
 }
 
 
@@ -980,7 +971,8 @@ Filter2D::create(string_view filtername, float width, float height)
         return new FilterBlackmanHarris2D(width, height);
     if (filtername == "sinc")
         return new FilterSinc2D(width, height);
-    if (filtername == "lanczos3" || filtername == "lanczos")
+    if (filtername == "lanczos3" || filtername == "lanczos"
+        || filtername == "nuke-lanczos6")
         return new FilterLanczos3_2D(width, height);
     if (filtername == "radial-lanczos3" || filtername == "radial-lanczos")
         return new FilterRadialLanczos3_2D(width, height);
