@@ -1,32 +1,6 @@
-/*
-  Copyright 2016 Larry Gritz and the other authors and contributors.
-  All Rights Reserved.
-
-  Redistribution and use in source and binary forms, with or without
-  modification, are permitted provided that the following conditions are
-  met:
-  * Redistributions of source code must retain the above copyright
-    notice, this list of conditions and the following disclaimer.
-  * Redistributions in binary form must reproduce the above copyright
-    notice, this list of conditions and the following disclaimer in the
-    documentation and/or other materials provided with the distribution.
-  * Neither the name of the software's owners nor the names of its
-    contributors may be used to endorse or promote products derived from
-    this software without specific prior written permission.
-  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
-  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
-  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
-  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
-  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
-  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
-  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
-  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
-  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-
-  (This is the Modified BSD License)
-*/
+// Copyright 2008-present Contributors to the OpenImageIO project.
+// SPDX-License-Identifier: BSD-3-Clause
+// https://github.com/OpenImageIO/oiio/blob/master/LICENSE.md
 
 
 // This implementation of thread_pool is based on CTPL.
@@ -44,10 +18,6 @@
 #    define _ENABLE_ATOMIC_ALIGNMENT_FIX /* Avoid MSVS error, ugh */
 #endif
 
-#if defined(WIN32)
-#    include <windows.h>
-#endif
-
 #include <exception>
 #include <functional>
 #include <future>
@@ -230,7 +200,7 @@ public:
             this->isDone = true;  // give the waiting threads a command to finish
         }
 
-#if defined(WIN32)
+#if defined(_WIN32)
         // When the static variable in default_thread_pool() is destroyed during DLL unloading,
         // the thread_pool destructor is called but the threads are already terminated.
         // So it is illegal to communicate with those other threads at this point.
@@ -294,12 +264,6 @@ public:
         return isPop;
     }
 
-    bool this_thread_is_in_pool() const
-    {
-        int* p = m_pool_members.get();
-        return p && (*p);
-    }
-
     void register_worker(std::thread::id id)
     {
         spin_lock lock(m_worker_threadids_mutex);
@@ -310,7 +274,7 @@ public:
         spin_lock lock(m_worker_threadids_mutex);
         m_worker_threadids[id] -= 1;
     }
-    bool is_worker(std::thread::id id)
+    bool is_worker(std::thread::id id) const
     {
         spin_lock lock(m_worker_threadids_mutex);
         return m_worker_threadids[id] != 0;
@@ -331,7 +295,6 @@ private:
         std::shared_ptr<std::atomic<bool>> flag(
             this->flags[i]);  // a copy of the shared ptr to the flag
         auto f = [this, i, flag /* a copy of the shared ptr to the flag */]() {
-            this->m_pool_members.reset(new int(1));  // I'm in the pool
             register_worker(std::this_thread::get_id());
             std::atomic<bool>& _flag = *flag;
             std::function<void(int id)>* _f;
@@ -343,11 +306,10 @@ private:
                     (*_f)(i);
                     if (_flag) {
                         // the thread is wanted to stop, return even if the queue is not empty yet
-                        this->m_pool_members
-                            .reset();  // I'm no longer in the pool
                         return;
-                    } else
+                    } else {
                         isPop = this->q.pop(_f);
+                    }
                 }
                 // the queue is empty here, wait for the next command
                 std::unique_lock<std::mutex> lock(this->mutex);
@@ -360,7 +322,6 @@ private:
                 if (!isPop)
                     break;  // if the queue is empty and this->isDone == true or *flag then return
             }
-            this->m_pool_members.reset();  // I'm no longer in the pool
             deregister_worker(std::this_thread::get_id());
         };
         this->threads[i].reset(
@@ -384,9 +345,8 @@ private:
     int m_size { 0 };           // Number of threads in the queue
     std::mutex mutex;
     std::condition_variable cv;
-    boost::thread_specific_ptr<int> m_pool_members;  // Who's in the pool
-    boost::container::flat_map<std::thread::id, int> m_worker_threadids;
-    spin_mutex m_worker_threadids_mutex;
+    mutable boost::container::flat_map<std::thread::id, int> m_worker_threadids;
+    mutable spin_mutex m_worker_threadids_mutex;
 };
 
 
@@ -453,10 +413,12 @@ thread_pool::push_queue_and_notify(std::function<void(int id)>* f)
 }
 
 
+
+/// DEPRECATED(2.1) -- use is_worker() instead.
 bool
 thread_pool::this_thread_is_in_pool() const
 {
-    return m_impl->this_thread_is_in_pool();
+    return is_worker();
 }
 
 
@@ -473,6 +435,14 @@ thread_pool::deregister_worker(std::thread::id id)
     m_impl->deregister_worker(id);
 }
 
+bool
+thread_pool::is_worker(std::thread::id id) const
+{
+    return m_impl->is_worker(id);
+}
+
+
+// DEPRECATED(2.1)
 bool
 thread_pool::is_worker(std::thread::id id)
 {
@@ -597,11 +567,27 @@ task_set::wait(bool block)
 
 
 
+// Helper function to keep track of the recursve depth of our use of the
+// thread pool. Call with the adjustment (i.e., parallel_recursive_depth(1)
+// to enter, parallel_recursive_depth(-1) to exit), and it will return the
+// new value. Call with default args (0) to just return the current depth.
+static int
+parallel_recursive_depth(int change = 0)
+{
+    thread_local int depth = 0;  // let's only allow one level of parallel work
+    depth += change;
+    return depth;
+}
+
+
+
 void
 parallel_for_chunked(int64_t start, int64_t end, int64_t chunksize,
                      std::function<void(int id, int64_t b, int64_t e)>&& task,
                      parallel_options opt)
 {
+    if (parallel_recursive_depth(1) > 1)
+        opt.maxthreads = 1;
     opt.resolve();
     chunksize = std::min(chunksize, end - start);
     if (chunksize < 1) {           // If caller left chunk size to us...
@@ -626,6 +612,7 @@ parallel_for_chunked(int64_t start, int64_t end, int64_t chunksize,
             ts.push(opt.pool->push(task, start, e));
         }
     }
+    parallel_recursive_depth(-1);
 }
 
 
@@ -637,11 +624,14 @@ parallel_for_chunked_2D(
     std::function<void(int id, int64_t, int64_t, int64_t, int64_t)>&& task,
     parallel_options opt)
 {
+    if (parallel_recursive_depth(1) > 1)
+        opt.maxthreads = 1;
     opt.resolve();
     if (opt.singlethread()
         || (xchunksize >= (xend - xstart) && ychunksize >= (yend - ystart))
         || opt.pool->very_busy()) {
         task(-1, xstart, xend, ystart, yend);
+        parallel_recursive_depth(-1);
         return;
     }
     if (ychunksize < 1)
@@ -660,6 +650,7 @@ parallel_for_chunked_2D(
             ts.push(opt.pool->push(task, x, xchunkend, y, ychunkend));
         }
     }
+    parallel_recursive_depth(-1);
 }
 
 
