@@ -1,32 +1,6 @@
-/*
-  Copyright 2018 Larry Gritz and the other authors and contributors.
-  All Rights Reserved.
-
-  Redistribution and use in source and binary forms, with or without
-  modification, are permitted provided that the following conditions are
-  met:
-  * Redistributions of source code must retain the above copyright
-    notice, this list of conditions and the following disclaimer.
-  * Redistributions in binary form must reproduce the above copyright
-    notice, this list of conditions and the following disclaimer in the
-    documentation and/or other materials provided with the distribution.
-  * Neither the name of the software's owners nor the names of its
-    contributors may be used to endorse or promote products derived from
-    this software without specific prior written permission.
-  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
-  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
-  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
-  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
-  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
-  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
-  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
-  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
-  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-
-  (This is the Modified BSD License)
-*/
+// Copyright 2008-present Contributors to the OpenImageIO project.
+// SPDX-License-Identifier: BSD-3-Clause
+// https://github.com/OpenImageIO/oiio/blob/master/LICENSE.md
 
 #include <OpenImageIO/dassert.h>
 #include <OpenImageIO/filesystem.h>
@@ -317,7 +291,7 @@ openVDB(const std::string& filename, const ImageInput* errReport)
     if (!Filesystem::is_regular(filename))
         return nullptr;
 
-    FILE* f = fopen(filename.c_str(), "r");
+    FILE* f = Filesystem::fopen(filename, "r");
     if (!f)
         return nullptr;
 
@@ -332,22 +306,27 @@ openVDB(const std::string& filename, const ImageInput* errReport)
     if (magic != OPENVDB_MAGIC)
         return nullptr;
 
-    static struct OpenVDBLib {
-        OpenVDBLib() { openvdb::initialize(); }
-        ~OpenVDBLib() { openvdb::uninitialize(); }
-    } sVDBLib;
-
-    VDBFile file(new io::File(filename));
+    const char* errhint = "Unknown error";
     try {
+        static struct OpenVDBLib {
+            OpenVDBLib() { openvdb::initialize(); }
+            ~OpenVDBLib() { openvdb::uninitialize(); }
+        } sVDBLib;
+
+        VDBFile file(new io::File(filename));
+
         file->open();
         if (file->isOpen())
             return file;
+
     } catch (const std::exception& e) {
-        errReport->error("Could not open '%s': %s", filename, e.what());
-        file.reset();  // Mark error reported
+        errReport->errorf("Could not open '%s': %s", filename, e.what());
+        return nullptr;
+    } catch (...) {
+        errhint = "Unknown exception thrown";
     }
-    if (file)
-        errReport->error("Could not open '%s': %s", filename, "Unknown error");
+
+    errReport->errorf("Could not open '%s': %s", filename, errhint);
     return nullptr;
 }
 
@@ -551,7 +530,7 @@ OpenVDBInput::open(const std::string& filename, ImageSpec& newspec)
         }
     } catch (const std::exception& e) {
         init();  // Reset to initial state
-        error("Could not open '%s': %s", filename, e.what());
+        errorf("Could not open '%s': %s", filename, e.what());
         return false;
     }
     m_name       = filename;
