@@ -1,32 +1,6 @@
-/*
-  Copyright 2009 Larry Gritz and the other authors and contributors.
-  All Rights Reserved.
-
-  Redistribution and use in source and binary forms, with or without
-  modification, are permitted provided that the following conditions are
-  met:
-  * Redistributions of source code must retain the above copyright
-    notice, this list of conditions and the following disclaimer.
-  * Redistributions in binary form must reproduce the above copyright
-    notice, this list of conditions and the following disclaimer in the
-    documentation and/or other materials provided with the distribution.
-  * Neither the name of the software's owners nor the names of its
-    contributors may be used to endorse or promote products derived from
-    this software without specific prior written permission.
-  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
-  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
-  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
-  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
-  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
-  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
-  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
-  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
-  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-
-  (This is the Modified BSD License)
-*/
+// Copyright 2008-present Contributors to the OpenImageIO project.
+// SPDX-License-Identifier: BSD-3-Clause
+// https://github.com/OpenImageIO/oiio/blob/master/LICENSE.md
 
 #pragma once
 
@@ -70,22 +44,40 @@ OIIO_PLUGIN_NAMESPACE_BEGIN
 
 namespace PNG_pvt {
 
+static void
+rderr_handler(png_structp png, png_const_charp data)
+{
+    ImageInput* inp = (ImageInput*)png_get_error_ptr(png);
+    if (inp && data)
+        inp->errorf("PNG read error: %s", data);
+}
+
+
+static void
+rdwarn_handler(png_structp png, png_const_charp data)
+{
+}
+
+
+
 /// Initializes a PNG read struct.
 /// \return empty string on success, error message on failure.
 ///
 inline const std::string
-create_read_struct(png_structp& sp, png_infop& ip)
+create_read_struct(png_structp& sp, png_infop& ip, ImageInput* inp = nullptr)
 {
-    sp = png_create_read_struct(PNG_LIBPNG_VER_STRING, NULL, NULL, NULL);
+    sp = png_create_read_struct(PNG_LIBPNG_VER_STRING, inp, rderr_handler,
+                                rdwarn_handler);
     if (!sp)
         return "Could not create PNG read structure";
 
+    png_set_error_fn(sp, inp, rderr_handler, rdwarn_handler);
     ip = png_create_info_struct(sp);
     if (!ip)
         return "Could not create PNG info structure";
 
     // Must call this setjmp in every function that does PNG reads
-    if (setjmp(png_jmpbuf(sp)))
+    if (setjmp(png_jmpbuf(sp)))  // NOLINT(cert-err52-cpp)
         return "PNG library error";
 
     // success
@@ -100,7 +92,7 @@ inline bool
 get_background(png_structp& sp, png_infop& ip, ImageSpec& spec, int& bit_depth,
                float* red, float* green, float* blue)
 {
-    if (setjmp(png_jmpbuf(sp)))
+    if (setjmp(png_jmpbuf(sp)))  // NOLINT(cert-err52-cpp)
         return false;
     if (!png_get_valid(sp, ip, PNG_INFO_bKGD))
         return false;
@@ -130,11 +122,16 @@ get_background(png_structp& sp, png_infop& ip, ImageSpec& spec, int& bit_depth,
 
 /// Read information from a PNG file and fill the ImageSpec accordingly.
 ///
-inline void
+inline bool
 read_info(png_structp& sp, png_infop& ip, int& bit_depth, int& color_type,
           int& interlace_type, Imath::Color3f& bg, ImageSpec& spec,
           bool keep_unassociated_alpha)
 {
+    // Must call this setjmp in every function that does PNG reads
+    if (setjmp(png_jmpbuf(sp)))  // NOLINT(cert-err52-cpp)
+        return false;
+
+    bool ok = true;
     png_read_info(sp, ip);
 
     // Auto-convert 1-, 2-, and 4- bit images to 8 bits, palette to RGB,
@@ -148,9 +145,8 @@ read_info(png_structp& sp, png_infop& ip, int& bit_depth, int& color_type,
     png_read_update_info(sp, ip);
 
     png_uint_32 width, height;
-    png_get_IHDR(sp, ip, &width, &height, &bit_depth, &color_type, NULL, NULL,
-                 NULL);
-
+    ok &= (bool)png_get_IHDR(sp, ip, &width, &height, &bit_depth, &color_type,
+                             nullptr, nullptr, nullptr);
 
     spec = ImageSpec((int)width, (int)height, png_get_channels(sp, ip),
                      bit_depth == 16 ? TypeDesc::UINT16 : TypeDesc::UINT8);
@@ -262,6 +258,8 @@ read_info(png_structp& sp, png_infop& ip, int& bit_depth, int& color_type,
         spec.attribute("oiio:UnassociatedAlpha", (int)1);
 
     // FIXME -- look for an XMP packet in an iTXt chunk.
+
+    return ok;
 }
 
 
@@ -275,7 +273,7 @@ read_into_buffer(png_structp& sp, png_infop& ip, ImageSpec& spec,
                  std::vector<unsigned char>& buffer)
 {
     // Must call this setjmp in every function that does PNG reads
-    if (setjmp(png_jmpbuf(sp)))
+    if (setjmp(png_jmpbuf(sp)))  // NOLINT(cert-err52-cpp)
         return "PNG library error";
 
 #if 0
@@ -310,7 +308,7 @@ inline const std::string
 read_next_scanline(png_structp& sp, void* buffer)
 {
     // Must call this setjmp in every function that does PNG reads
-    if (setjmp(png_jmpbuf(sp)))
+    if (setjmp(png_jmpbuf(sp)))  // NOLINT(cert-err52-cpp)
         return "PNG library error";
 
     png_read_row(sp, (png_bytep)buffer, NULL);
@@ -393,7 +391,7 @@ create_write_struct(png_structp& sp, png_infop& ip, int& color_type,
         return "Could not create PNG info structure";
 
     // Must call this setjmp in every function that does PNG writes
-    if (setjmp(png_jmpbuf(sp)))
+    if (setjmp(png_jmpbuf(sp)))  // NOLINT(cert-err52-cpp)
         return "PNG library error";
 
     // success
@@ -614,7 +612,7 @@ write_info(png_structp& sp, png_infop& ip, int& color_type, ImageSpec& spec,
 inline bool
 write_row(png_structp& sp, png_byte* data)
 {
-    if (setjmp(png_jmpbuf(sp))) {
+    if (setjmp(png_jmpbuf(sp))) {  // NOLINT(cert-err52-cpp)
         //error ("PNG library error");
         return false;
     }
@@ -630,7 +628,7 @@ inline void
 finish_image(png_structp& sp, png_infop& ip)
 {
     // Must call this setjmp in every function that does PNG writes
-    if (setjmp(png_jmpbuf(sp))) {
+    if (setjmp(png_jmpbuf(sp))) {  // NOLINT(cert-err52-cpp)
         //error ("PNG library error");
         return;
     }
